--Ryan Hoskin  - XXX-XX-3343 - Lab Section W:2-5
--Siraj Husain - XXX-XX-2911 - Lab Section W:2-5 
--Computer Science 161 - Assignment 1 - Datapath

-- Datapath

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity Datapath is
    port(
	    rst : in std_logic;
	    clk : in std_logic;
	
	    --control signals				
		alu_src_A 	: in std_logic;				  	
		alu_src_B 	: in std_logic_vector( 1 downto 0 );
		
	    -- output from the ALU control shown in Figure 5.33 (ALUop)
		alu_control : in std_logic_vector( 2 downto 0 );
		reg_write 	: in std_logic;	
		reg_dst 	: in std_logic;
		pc_source 	: in std_logic_vector( 1 downto 0 );
	    
		-- output from the or gate, combining PCWriteCond and PCWrite
	    pc_load 	: in std_logic;
		i_or_d 		: in  std_logic;
		mem_read 	: in std_logic;
		mem_write 	: in std_logic;
		mem_to_reg 	: in std_logic;
		IR_write 	: in  std_logic;
	        
	    -- output to controller
		ins_31_26	: out std_logic_vector( 5 downto 0 );
	    -- output to ALU control
	    ins_5_0   	: out std_logic_vector( 5 downto 0 );
	    -- output to PC write logic
	    alu_zero   	: out std_logic;
	
	    -- used for testing
	    testing_alu_result    : out std_logic_vector(31 downto 0);
	    testing_mem_data      : out std_logic_vector(31 downto 0);
	    testing_read_data_1   : out std_logic_vector(31 downto 0);
	    testing_read_data_2   : out std_logic_vector(31 downto 0)
    );
    
end Datapath;

-- purpose: put all the components together
architecture bhv of Datapath is

------------------------------------------------------------------------
component Memory
    port( 
	rst        : in  std_logic;						--reset signal
    clk		   : in  std_logic;        				--clock signal
	rd         : in  std_logic;						--read data
	wr         : in  std_logic;						--write register
	addr       : in  std_logic_vector (31 downto 0);--address
	in_data    : in  std_logic_vector (31 downto 0);--input value
	out_data   : out std_logic_vector (31 downto 0)	--output value
	);
end component;															
------------------------------------------------------------------------

-- data read from Memory, connect to input of IR and MDR
signal mem_data   : std_logic_vector(31 downto 0);

------------------------------------------------------------------------
component ALU
    port(
	input_a    : in std_logic_vector( 31 downto 0);
	input_b    : in std_logic_vector( 31 downto 0 );
	op         : in std_logic_vector( 2 downto 0 );	
	zero       : out std_logic;
	result     : out std_logic_vector( 31 downto 0 )
	);
end component; 
------------------------------------------------------------------------

--output from alu to aluout register and to mux 6
signal alu_out		: std_logic_vector (31 downto 0); 

------------------------------------------------------------------------
component Mux4 is
port(							
	I0: 	in std_logic_vector(31 downto 0);		--input 3
	I1: 	in std_logic_vector(31 downto 0);		--input 2
	I2: 	in std_logic_vector(31 downto 0);		--input 1
	I3: 	in std_logic_vector(31 downto 0);		--input 0
	S:	in std_logic_vector(1 downto 0); 			--select signal
	O:	out std_logic_vector(31 downto 0) 			--output signal
);
end component; 
------------------------------------------------------------------------

--output from 32-bit 4:1 mux 5
signal mux45_out	: std_logic_vector (31 downto 0);

------------------------------------------------------------------------
component Mux3 is
port(														  
	I0: 	in std_logic_vector(31 downto 0);		--input 2
	I1: 	in std_logic_vector(31 downto 0);		--input 1
	I2: 	in std_logic_vector(31 downto 0);		--input 0
	S:		in std_logic_vector(1 downto 0); 		--select signal
	O:		out std_logic_vector(31 downto 0) 		--output signal
);
end component; 	   		   
------------------------------------------------------------------------

--output from 32-bit 3:1 mux 6
signal mux36_out	: std_logic_vector (31 downto 0);

------------------------------------------------------------------------
component mux232 is
port(		
	I0: 	in std_logic_vector(31 downto 0);		--input 1
	I1: 	in std_logic_vector(31 downto 0);		--input 0
	S:		in std_logic;					 		--select signal
	O:		out std_logic_vector(31 downto 0) 		--output signal
);
end component; 															
------------------------------------------------------------------------

--output from 32-bit 2:1 mux 4
signal mux24_out	: std_logic_vector (31 downto 0);
--output of 32-bit 2:1 mux 3
signal mux23_out	: std_logic_vector (31 downto 0); 
--memory address, output of 32-bit 2:1 mux 1
signal address  	: std_logic_vector(31 downto 0);

------------------------------------------------------------------------
component Mux25 is
port(		
	I0: 	in std_logic_vector(4 downto 0);		--input 1
	I1: 	in std_logic_vector(4 downto 0);		--input 0
	S:		in std_logic;					 		--select signal
	O:		out std_logic_vector(4 downto 0) 		--output signal
);
end component; 															
------------------------------------------------------------------------

--output of 5-bit 2:1 mux 2
signal mux22_out	: std_logic_vector (4 downto 0);

------------------------------------------------------------------------
component shifter32 is													
port(	
	I:		in std_logic_vector (31 downto 0); 	--input signal
	clk:		in std_logic; 						--clock signal 
	O:			out std_logic_vector (31 downto 0) 	--output signal
);
end component;															
------------------------------------------------------------------------

--output of 32-bit shifter (shift left 2)
signal shifter32_out : std_logic_vector (31 downto 0);  

------------------------------------------------------------------------
component shifter26 is
port(	I:		in std_logic_vector (25 downto 0);
	clk:		in std_logic;  
	O:		out std_logic_vector (27 downto 0)
);
end component;														
------------------------------------------------------------------------

--output of 28-bit shifter (shift left 2)
signal shifter26_out : std_logic_vector (27 downto 0);

------------------------------------------------------------------------
component sx is
port(
	I: 	in std_logic_vector(15 downto 0); 			--input signal
	O:	out std_logic_vector(31 downto 0) 			--output signal
);
end component;															
------------------------------------------------------------------------  

--output of sign extender
signal sx_out	: std_logic_vector (31 downto 0);

------------------------------------------------------------------------
component register_ld is
  port(
      rst   : in std_logic;
      clk   : in std_logic;
      ld  : in std_logic;
      i	    : in std_logic_vector (31 downto 0);
      o     : out std_logic_vector (31 downto 0)
      );
end component;													
------------------------------------------------------------------------

--output of program counter 'register'
signal pc_out		: std_logic_vector (31 downto 0); 

signal ir_out		: std_logic_vector (31 downto 0);

------------------------------------------------------------------------
component reg
  port(
      rst   : in std_logic;
      clk   : in std_logic;
      input : in std_logic_vector (31 downto 0);
      output: out std_logic_vector (31 downto 0)
      );
end component;													
------------------------------------------------------------------------

--output of A register
signal a_out		: std_logic_vector (31 downto 0);	 
--data to write to memory, output of B register
signal write_data : std_logic_vector(31 downto 0);
--output of ALUout register
signal aluout_out	: std_logic_vector (31 downto 0);
--output of memory data register
signal mdr_out		: std_logic_vector (31 downto 0);

------------------------------------------------------------------------	

component reg_file is
    port ( 
	rst         : in std_logic;
	clk         : in std_logic;
	rw   : std_logic;
	rr1  : in std_logic_vector(4 downto 0);
	rr2  : in std_logic_vector(4 downto 0);
	wrr   : in std_logic_vector(4 downto 0);
	wrd  : in std_logic_vector(31 downto 0);
	rd1 : out std_logic_vector(31 downto 0);
	rd2 : out std_logic_vector(31 downto 0)
	);		
end component;
													
------------------------------------------------------------------------

signal rd1_out	: std_logic_vector (31 downto 0);			--read data 1
signal rd2_out	: std_logic_vector (31 downto 0);		 	--read data 2	   

------------------------------------------------------------------------

constant four	: std_logic_vector (31 downto 0) := "00000000000000000000000000000100";	--static value 4
signal j_a		: std_logic_vector (31 downto 0);	 		--jump address signal  


signal ins_25_0  : std_logic_vector (25 downto 0);			--signals for instructions
signal ins_25_21 : std_logic_vector (4 downto 0);
signal ins_20_16 : std_logic_vector (4 downto 0);
signal ins_15_11 : std_logic_vector (4 downto 0);	  
signal ins_15_0  : std_logic_vector (15 downto 0);

------------------------------------------------------------------------

begin  --Begin behavior of Datapath	  

	PC		: register_ld 	port map (rst, clk, pc_load, mux36_out, pc_out); 
	MUX_1	: mux232 		port map (pc_out, aluout_out, i_or_d, address);	
	U_MEM 	: Memory 		port map (rst, clk, mem_read, mem_write, address, write_data, mem_data);
	I_REG	: register_ld	port map (rst, clk, IR_write, mem_data, ir_out);
	MDR		: reg			port map (rst, clk, mem_data, mdr_out);
	MUX_2f	: mux25			port map (ins_20_16, ins_15_11, reg_dst, mux22_out); 
	MUX_3	: mux232		port map (aluout_out, mdr_out, mem_to_reg, mux23_out);	
	RF		: reg_file		port map (rst, clk, reg_write, ins_25_21, ins_20_16, mux22_out, mux23_out, rd1_out, rd2_out);
	A		: reg			port map (rst, clk, rd1_out,  a_out);
	B		: reg			port map (rst, clk, rd2_out, write_data); 		 
	S_X		: sx			port map (ins_15_0, sx_out);
	SL32	: shifter32		port map (sx_out, clk, shifter32_out);
	MUX_4	: mux232		port map (pc_out, a_out, alu_src_A, mux24_out);
	MUX_5	: mux4			port map (write_data, four, sx_out, shifter32_out, alu_src_B, mux45_out);
	ALU_1	: alu			port map (mux24_out, mux45_out, alu_control, alu_zero, alu_out); 
	ALU_O	: reg			port map (rst, clk, alu_out, aluout_out);
	SL26	: shifter26		port map (ins_25_0, clk, shifter26_out);
	blah	: j_a (31 downto 0) <= pc_out (31 downto 28) & shifter26_out (27 downto 0);	--concatenate jump address
	MUX_6  	: mux3			port map (alu_out, aluout_out, j_a, pc_source, mux36_out);	
	
	ins1	: ins_25_0  <= ir_out(25 downto 0);				  --parse instruction code
	ins2	: ins_31_26 <= ir_out(31 downto 26);
	ins3	: ins_25_21 <= ir_out(25 downto 21);
	ins4	: ins_20_16 <= ir_out(20 downto 16);
	ins5	: ins_15_11 <= ir_out(15 downto 11);	  
	ins6	: ins_5_0   <= ir_out(5 downto  0);
	ins7	: ins_15_0  <= ir_out(15 downto  0);
  
	cpy1	: testing_alu_result <= alu_out;				  --connect test signals
  	cpy2	: testing_mem_data <= mem_data;
  	cpy3	: testing_read_data_1 <= rd1_out;
  	cpy4	: testing_read_data_2 <= rd2_out;
	  
end bhv;  
-----------------------------------------------------------------------

